include #/bearun/msc/mess.lan

// --------------------------------------------------------------------------------
/*
Restrictions for external layers
--------------------------------
If the external layer is not loaded or readonly, then not all layer attributes can 
be changed and not all layer operations are possible.
Internal sublayers of external layers behave like the external layer, to which
they belong.

 --------------------------------------------------------------------   
| Feature        | layer is not loaded     | layer is readonly       |   
| -------------------------------------------------------------------|   
| active         | can't be changed        | can't be changed        |
| locked         | can't be changed        | can't be changed        |
| color          | can't be changed        | can be changed          | 
|                |                         |                         | 
| edit, create,  | not possible            | not possible            | 
| and delete     |                         |                         | 
| objects in     |                         |                         |
| layer          |                         |                         |
|                |                         |                         |
| readonly       | can't be changed        | can be changed, if UNIX-| 
|                |                         | write access            |
| extern/intern  | can't be changed        | can't be changed        | 
| is_a           | can't be changed        | can't be changed        | 
| name           | can't be changed        | can't be changed        | 
| delete layer   | not possible            | not possible            | 
| undelete layer | not possible            | not possible            | 
| move layer     | not possible            | not possible            | 
| move objects   | not possible            | not possible            | 
| set userattribs| not possible            | not possible            | 
|                |                         |                         |
 --------------------------------------------------------------------   
*/

// --------------------------------------------------------------------------------
function check_external_layers ( layer_in, check_if_loaded, check_access_rights )

   // This function is used in the all situations described in the above table.

   ok = true

   layer = layer_in

   if not layer_in.is_layer then return(ok) end

   // --- check if layer is internal sublayer of external layer
   if not layer.external then
      layers = {} // store all external layers, but not top
      for i in top.list_user 
         if i.is_layer and i.external and i != top then layers = layers + i end
      end
      for i in layers // for all external layers
         for j in i.list_user
            if j = layer_in then layer = i end // layer_in is sublayer
         end
      end
   end

   // --- we are only interested in external layers, which are no top_models:
   is_external = layer.external and layer != top

   if (layer.external and layer != top) then
      if (check_if_loaded and not layer.loaded) then
         ok = false
         display_error (translate_message(run_err_please_load_first, layer.name))
      end
      if (ok and check_access_rights and layer.loaded) then
         ok = not layer.read_only and file_access (layer.filename, "w")
         if not ok then         
            display_error (translate_message(run_err_ext_lay_no_checkout, layer.name)) 
         end
      end
   end

   return (ok)

end

