// *************************************************************
// Created    :  Christian Manitz
// 
// - AQL program to start, call and stop a help viewer.
// - Define viewer tool
// 
// *************************************************************

include #/bearun/msc/mess.lan
include #/bearun/udx/browser_config.inc

var Var_ProgramName = "#/bearun/aql/call_help.inc"

var Var_BrowserHelp
var Var_BrowserDocu
var Var_ProgramHelp
var Var_ProgramDocu
var Var_ProgramOptionHelp
var Var_ProgramOptionDocu
var Var_DocSuffix
var Var_Language
var Var_IsWindows
var Var_InstPathHelp
var Var_InstPathDocu
var UseDefaultBrowser


//------------------------------------------------

  Var_IsWindows = is_windows_host (program.host)
  Var_Language  = index (program.languages, pos (program.language_strings, program.language))


/************************************************************/
/* Define the kind of viewer tool                           */
/************************************************************/
function SetViewerTypeHelp ()

// - Read browser configuration file
// - Initialize global variables
  UseDefaultBrowser = false
  ConfigData = ReadBrowserConfig ()
  if (not empty (ConfigData)) then
    Var_BrowserHelp  = ConfigData.el_1
    BrowserOption    = ConfigData.el_2
    WinBrowser       = ConfigData.el_3
    BrowserProgram   = ConfigData.el_4
    Var_InstPathHelp = ConfigData.el_8
    Var_ProgramOptionHelp = ""

    MessWrongBrowser = translate_message (run_war_cant_show_ol_help) + "\n\n" +
                       translate_message (run_war_wrong_browser_config) + "\n" +
                       GetBrowserConfig () + "\n\n" +
                       translate_message (run_war_select_browser)
    switch ( Var_BrowserHelp )
      case "HTML" :
          if ( Var_IsWindows ) then
            Var_ProgramHelp = program.directory + "/tools/dde_command NODEBUG "
            changeSlashToBackslash (Var_ProgramHelp)
            switch (BrowserOption)
              case "DEFAULT": Var_ProgramOptionHelp = "DEFAULT \"file:///"
                              UseDefaultBrowser     = true
              case "PATH"   : Var_ProgramOptionHelp = "\"" + WinBrowser + "\" \"file:///"
              case "INST"   : Var_ProgramOptionHelp = "\"" + WinBrowser + BrowserProgram + "\" \"file:///"
              otherwise     : display_error (MessWrongBrowser)
                              return (false)
            end
          else
            switch (BrowserOption)
              case "PATH"   : Var_ProgramHelp = "netscape "
                              Var_ProgramOptionHelp = "-raise -remote 'openFile "     // Netscape options
              case "INST"   : Var_ProgramHelp = BrowserProgram + " "
                              Var_ProgramOptionHelp = "-raise -remote 'openFile "     // Corresponding options??
              otherwise     : display_error (MessWrongBrowser)
                              return (false)
            end
          end  // if ( Var_IsWindows )
          Var_DocSuffix   = "." + lowercase (Var_BrowserHelp)
      case "FRAME": 
          Var_ProgramHelp = program.directory + "/tools/d2toframe.exe "
          switch (BrowserOption)
            case "DEFAULT": Var_ProgramOptionHelp = "DEFAULT "
            case "INST"   : Var_ProgramOptionHelp = BrowserProgram + " "
            otherwise     : display_error (MessWrongBrowser)
                            return (false)
          end
          Var_DocSuffix   = ""
      otherwise   : display_error (MessWrongBrowser)
                    return (false)
    end  // switch ( Var_BrowserHelp )
  else
    MessNoBrowser = translate_message (run_war_cant_show_ol_help) + "\n\n" +
                    translate_message (run_war_no_browser_config) + "\n" +
                    GetBrowserConfig () + "\n\n" +
                    translate_message (run_war_no_select_browser)
    display_error (MessNoBrowser)
    return (false)
  end  // if (not empty (ConfigData))

  return (true)

end  // SetViewerTypeHelp


//---------------------------------------------------------
function SetViewerTypeDocu ()

// - Read browser configuration file
// - Initialize global variables

  ConfigData = ReadBrowserConfig ()
  if (not empty (ConfigData)) then
    Var_BrowserDocu  = ConfigData.el_5
    BrowserOption    = ConfigData.el_6
    BrowserProgram   = ConfigData.el_7
    Var_InstPathDocu = ConfigData.el_9
    Var_ProgramOptionDocu = ""

    MessWrongBrowser = translate_message (run_war_cant_show_ol_help) + "\n\n" +
                       translate_message (run_war_wrong_browser_config) + "\n" +
                       GetBrowserConfig () + "\n\n" +
                       translate_message (run_war_select_browser)

    switch ( Var_BrowserDocu )
      case "FRAME": 
          Var_ProgramDocu = program.directory + "/tools/d2toframe.exe "
          switch (BrowserOption)
            case "DEFAULT": Var_ProgramOptionDocu = "DEFAULT "
            case "INST"   : Var_ProgramOptionDocu = BrowserProgram + " "
            otherwise     : display_error (MessWrongBrowser)
                            return (false)
          end
          Var_DocSuffix   = ""
      case "PDF" :
          if ( Var_IsWindows ) then
            Var_ProgramDocu = program.directory + "/tools/dde_command NODEBUG PDFREADER"
            changeSlashToBackslash (Var_ProgramDocu)
            switch (BrowserOption)
              case "DEFAULT": Var_ProgramOptionDocu = " "
              case "PATH"   : Var_ProgramOptionDocu = "PATH "
                              display_error ("Not yet implemented!")
              case "INST"   : Var_ProgramOptionDocu = BrowserProgram + " "
                              display_error ("Not yet implemented!")
              otherwise     : display_error (MessWrongBrowser)
                              return (false)
            end
          else
            switch (BrowserOption)
              case "PATH"   : Var_ProgramDocu = "/usr/adobe/Acrobat4.0/bin/acroread "
                              Var_ProgramOptionDocu = ""         // no options necessary
              case "INST"   : Var_ProgramDocu = BrowserProgram + " "
                              Var_ProgramOptionDocu = ""
              otherwise     : display_error (MessWrongBrowser)
                              return (false)
            end
          end  // if ( Var_IsWindows )
          Var_DocSuffix   = "." + lowercase (Var_BrowserDocu)
      otherwise   : display_error (MessWrongBrowser)
                    return (false)
    end  // switch ( Var_BrowserDocu )
  else
    MessNoBrowser = translate_message (run_war_cant_show_ol_docu) + "\n\n" +
                    translate_message (run_war_no_browser_config) + "\n" +
                    GetBrowserConfig () + "\n\n" +
                    translate_message (run_war_no_select_browser)
    display_error (MessNoBrowser)
    return (false)
  end  // if (not empty (ConfigData))

  return (true)

end  // SetViewerTypeDocu


//-------------------------------------------------
function CallBrowserHelp (UseOption, FileNameEtc)
//-------------------------------------------------
// Call the browser
// Function will be called from outside too!

// INPUT:
//   UseOption      boolean             true:  use global var. for options
//                                      false: start browser directly with filename
//   FileNameEtc    string              name of file to show with browser
// RETURN:
//                  boolean             true:  system command was ok
//                                      false: system command reports an error
  if UseDefaultBrowser = true then
     Ok = open_url (FileNameEtc, true)
     if Ok = true then
        Ok = 0
     else
        Ok = 1
     end
  else
     if ( UseOption ) then
       Ok = system (Var_ProgramHelp + Var_ProgramOptionHelp + FileNameEtc)
     else
       Ok = system (Var_ProgramHelp + FileNameEtc)
     end
  end

  if (Ok != 0) then
    return (false)
  else
    return (true)
  end

end  // CallBrowserHelp


/************************************************************/
/* Open a help file and jump to the tag                     */
/************************************************************/

//-----------------------------
function DefineTag (Number)
//-----------------------------
// Build the tag (to jump to) combining the unique UDx number
// INPUT:
//   Number         integer             the unique number of system nodes
// RETURN:
//   string: Tag to jump to

  return ("D2" + string (Number,,,"X"))

end  // DefineTag


//-----------------------------
function DefineUDxTag (IdGrp)
//-----------------------------
// Build the tag (to jump to) combining the unique UDx number

// INPUT:
//   IdGrp          group of integers   portions of the unique number of UDx
// RETURN:
//   string: Tag to jump to

  TagNr = "D2"             // Tag-Nr fuer Online-Help
  for i in IdGrp do
    TagNr = TagNr + string (i,,,"X") + "_"
  end
  TagNr = TagNr - "*_"
  return (TagNr)

end  // DefineUDxTag


//-----------------------------
function SplitPath (FullPath)
//-----------------------------
// Cut the filename from a full pathname

// INPUT:
//   FullPath       string              pathname, e.g. /usr/people/xxx.udo
// RETURN:
//   - string: path without filename, incl. "/" or "\"
//   - invalid: no path found

  i = len (FullPath)
  if (i > 0) then
    while ((i>0)    and
           (substr (FullPath, i, i) != "/")    and
           (substr (FullPath, i, i) != "\\"))  do
      i = i - 1
    end
    if (i < len (FullPath)) then
      if (i > 0) then
        return (substr (FullPath, 1, i))
      end
    end
  end  // if (i > 0)
  return (invalid)

end  // SplitPath


//----------------------------------------------------------
function CheckInstPath (PathName, BrowserTool, HelpOrDocu)
//----------------------------------------------------------
// - Extend filename
// - Check its existance: 
//   Condition: The installation path itself does exist
//   Check:     does the installation path exist for the current language?
//              If not: take english instead (unchecked!)

// INPUT:
//   PathName       string              Pathname of online help or docu; from 'browser.cfg'
//                                      
//   BrowserTool    string              Browser of online help or docu
//   HelpOrDocu     string              "help": use 'BrowserTool' as subdirectory (<= V5.7)
//                                      "docu": no subdirectory                   (>= V2001: same as EUKLID)
// RETURN:
//   SystemPath     string              The probably existing pathname

  SystemPath = FileExtendAll (PathName)
  changeBackslashToSlash (SystemPath)                                    // to keep 'existd' happy
  LastChar = substr (SystemPath, len (SystemPath), len (SystemPath))
  if (not (LastChar = "/" )) then                                        // sometimes a '/' is added (e.g. v:\runtime\)
    SystemPath = SystemPath + "/"                                        // InternetExplorer doesn't run with '//'
  end

  if (HelpOrDocu = "help") then
    TmpPath = SystemPath + lowercase (BrowserTool) + "/" + Var_Language  // <= V5.7
  else
    TmpPath = SystemPath + Var_Language                                  // >= V2001: same as EUKLID
  end
  if (not existd (TmpPath)) then                                         // Does special language exist? (e.g. Jp)
    // SystemPath = SystemPath + lowercase (BrowserTool) + "/En/"        // Use english instead  <= 5.7
    SystemPath = SystemPath + "/En/"                                     // Use english instead
    if (not existd (SystemPath)) then
      SystemPath = TmpPath + "/"
    end
  else
    SystemPath = TmpPath + "/"
  end
  return (SystemPath)

end  // CheckInstPath


//------------------------------------------
function JumpToItem ()
//------------------------------------------  entry from the software program

  ActionNode = aql_arg_0           // action | object | enum_field
  ParamNode  = aql_arg_1           // parameter | property

  if (not SetViewerTypeHelp ()) then
    return (false)
  end

  if (len (Var_InstPathHelp) > 0) then
    SystemHelpPath = CheckInstPath (Var_InstPathHelp, Var_BrowserHelp, "help")
  else
    SystemHelpPath = ""
  end

  UdxSuffix = "_" + Var_Language + "." + lowercase (Var_BrowserHelp)
  
  if (valid (ActionNode)) then                                                       // No help available on action group
    if (ActionNode.type = "actiongroup"  or  ActionNode.type = "objectgroup") then   // (1st column)
      display_error (translate_message (run_err_no_help_UDM))
      return ()
    end
  end

  IsUDx = false
  TagSetMyself = false
  if (valid (ParamNode)) then                                                   // Parameter icon was clicked
    if (valid (ParamNode.user_defined)  and  ParamNode.user_defined ) then      // UDx
      IsUDx = true
      if (valid (ParamNode.frame_user_help)       and
          not empty (ParamNode.frame_user_help)   and
          len (ParamNode.frame_user_help.el_1) > 0) then
        DocName = ParamNode.frame_user_help.el_1
        TagName = ParamNode.frame_user_help.el_2
      else
        if (valid (ActionNode.frame_user_help)       and
            not empty (ActionNode.frame_user_help)   and
            len (ActionNode.frame_user_help.el_1) > 0) then
          DocName = ActionNode.frame_user_help.el_1
          TagName = DefineUDxTag (ParamNode.expanded_number)
          TagSetMyself = true
        else
          DocName = ActionNode.name
          TagName = DefineUDxTag (ParamNode.expanded_number)
          TagSetMyself = true
        end
      end  // if (valid (ParamNode. ...
    else                                        // system defined parameter
      TagName = DefineTag (ParamNode.number)
    end  // if (valid (ParamNode. ...
  else                                          // Action icon was clicked
    if (valid (ActionNode.user_defined)  and  ActionNode.user_defined ) then   // UDx
      IsUDx = true
      if (valid (ActionNode.frame_user_help)       and
          not empty (ActionNode.frame_user_help)   and
          len (ActionNode.frame_user_help.el_1) > 0) then
        DocName = ActionNode.frame_user_help.el_1
        TagName = ActionNode.frame_user_help.el_2
      else
        DocName = ActionNode.name
        TagName = DefineUDxTag (ActionNode.expanded_number)
        TagSetMyself = true
      end
    else                                        // system defined action
      TagName = DefineTag (ActionNode.number)
    end  // if (valid (ActionNode. ...
  end  // if (valid (ParamNode))

  if ( IsUDx ) then
    FileName = FileExtendAll (DocName)
    if ( not (chr (index (FileName, 1))  in {"/", "\\"}) ) then
      PathName = SplitPath (ActionNode.file)
      if (valid (PathName)) then
        FileName = PathName + DocName          // pathname of UDx
      else
        FileName = ActionNode.file + DocName
      end
      FileName = FileExtendAll (FileName)
    end  // if ( not (chr (index ...

    FileName = FileName + UdxSuffix
    if ( not existf (FileName) ) then            // perhaps in system help?
      switch (ActionNode.type)
        case "action"    : SystemFileName = SystemHelpPath + "actions/"    + DocName + UdxSuffix
        case "object"    : SystemFileName = SystemHelpPath + "data_types/" + DocName + UdxSuffix
        case "value"     : SystemFileName = SystemHelpPath + "data_types/" + DocName + UdxSuffix
        otherwise        : display_error ("Wrong node type:  " + ActionNode.type + "%%No Online Help available")
                           return
      end
      if (existf (SystemFileName)) then
        FileName = SystemFileName
      end
    else
      if ( TagSetMyself ) then
        TagName = ""     // it's a real user file, probably without our tag "D2..."
      end
    end  // if ( not existf (FileName)
  else
    switch (ActionNode.type)
      case "enum_field": if (ActionNode.name = "abs") then               // keyboard icon in 4th column = method to create abs. object
                           if (valid (program.active_par_or_prop)) then              // i.e. angle, length, string, number, prop, view
                             if (valid (program.active_par_or_prop.type.ref_to.name)) then                      // jump to object file
                               FileName = SystemHelpPath + "data_types/" + program.active_par_or_prop.type.ref_to.name
                               TagName = ""
                             end
                           else
                             display_error (translate_message (run_war_cant_show_ol_help) + translate_message (run_war_this_obj_type))
                           end                             
                         else
                           FileName = SystemHelpPath + "data_types/EnumerationTypes"
                         end
      case "action"    : FileName = SystemHelpPath + "actions/" + ActionNode.name
      case "object"    : FileName = SystemHelpPath + "data_types/" + ActionNode.name
      case "value"     : FileName = SystemHelpPath + "data_types/" + ActionNode.name
      otherwise        : display_error ("Wrong node type:  " + ActionNode.type + "%%No Online Help available")
                         return
    end
    FileName = FileName + Var_DocSuffix
  end  // if ( IsUDx )

  if ( not existf (FileName) ) then
    display_error (translate_message (run_err_file_not_exist) + "\n\n" + FileName)
    return ()
  end

  SysOk = 0
  switch ( Var_BrowserHelp )
    case "FRAME": CallCmd = Var_ProgramHelp + Var_ProgramOptionHelp + "Jump " +
                            FileName + " " + TagName
                  SysOk = CallBrowserHelp (true, "Jump " + FileName + " " + TagName)
    case "HTML" : if (len (TagName) = 0) then
                    Tag = ""                 // Netscape pops a message if tag doesn't exist
                  else
                    Tag = "#" + TagName
                  end
                  if (Var_IsWindows) then
                    if ( not CallBrowserHelp (true, FileName + Tag + "\"")) then      // try to pop browser
                      SysOk = CallBrowserHelp (false, "\"" + FileName + Tag + "\"&")  // error, try to start browser
                    end
                  else
                    if ( not CallBrowserHelp (true, FileName + Tag + "'")) then
                      SysOk = CallBrowserHelp (false, FileName + Tag + "&")
                    end
                  end                    
  end

  if ( not SysOk ) then
    display_error (translate_message (run_err_problem_browser) + "\n\n" + Var_ProgramHelp)
  end

end  // JumpToItem


/************************************************************/
/* Call the documentation                                   */
/************************************************************/

function CallDocumentation ()
//------------------------------------------  entry from the software program

  if (not SetViewerTypeDocu ()) then
    return (false)
  end

  SystemDocuPath = CheckInstPath (Var_InstPathDocu, Var_BrowserDocu, "docu")

  FileName = SystemDocuPath + "od_start" + Var_DocSuffix
  if (Var_IsWindows) then
    changeSlashToBackslash (FileName)
  end
  if ( not existf (FileName) ) then
    display_error (translate_message (run_err_file_not_exist) + "\n\n" + FileName)
    return ()
  end

  SysOk = 0
  switch ( Var_BrowserDocu )
    case "FRAME": CallCmd = Var_ProgramDocu + Var_ProgramOptionDocu + "StartDocu " + FileName
                  SysOk = system (CallCmd)
    case "PDF"  : CallCmd = Var_ProgramDocu + Var_ProgramOptionDocu + FileName + " &"
                  SysOk = system (CallCmd)
  end

  if ( SysOk != 0 ) then
    display_error (translate_message (run_err_problem_browser) + "\n\n" + Var_ProgramDocu)
  end

end  // CallDocumentation 
